#!/bin/bash
# saveconfig - create KNOPPIX configuration floppy
# (C) Klaus Knopper Mar 2002

PATH="/bin:/sbin:/usr/bin:/usr/sbin"
export PATH


# Get root
[ "`id -u`" != "0" ] && exec sudo "$0" "$@"

TMP="/tmp/saveconfig.tmp$$"

bailout(){
rm -f "$TMP"
exit 0
}

DIALOG="dialog"
[ -n "$DISPLAY" ] && [ -x /usr/bin/gdialog ] && DIALOG="gdialog"

trap bailout 1 2 3 15

# LANGUAGE etc.
[ -f /etc/sysconfig/knoppix ] && . /etc/sysconfig/knoppix
[ -z "$LANG" ] && export LANG
[ -z "$LANGUAGE" ] && export LANGUAGE
[ -z "$CHARSET" ] && export CHARSET

# Language-dependent Messages
case "$LANGUAGE" in
de*)
TITLE1="KNOPPIX Konfigurationsfloppy anlegen"
MESSAGE1="Konfigurationsdateien auswhlen:"
MESSAGE2="Bitte legen Sie jetzt eine leere DOS- oder ext2-formatierte, schreibbare Diskette ein."
MESSAGE3="Speichere auf Diskette..."
E1="Persnliche Einstellungen (Desktop, Programme)"
E2="Systemweite Einstellungen (Netzwerk, Drucker, Bildschirm etc.)"
SUCCESS="Die KNOPPIX-Konfigurationsdiskette wurde erfolgreich angelegt. Ihre Konfigurationsdateien werden beim nchsten KNOPPIX-Bootvorgang in die Ramdisk restauriert, wenn Sie im Bootscreen \"knoppix floppyconf\" angeben."
ERROR="Leider konnte die KNOPPIX-Konfigurationsdiskette NICHT angelegt werden:"
;;
*)
TITLE1="Create KNOPPIX configuration floppy"
MESSAGE1="Chose type of configuration files:"
MESSAGE2="Please insert an empty DOS- or ext2-formatted, writable floppy disk."
MESSAGE3="Saving to floppy..."
E1="Personal configuration (desktop, programs)"
E2="System configuration (network, printer, screen etc.)"
SUCCESS="Creation of KNOPPIX configuration floppy was successful. Your configuration files will be reinstalled to the ramdisk on next KNOPPIX boot if you specify \"   knoppix floppyconf\" at the boot prompt."
ERROR="The KNOPPIX configuration floppy could NOT be created:"
;;
esac

$DIALOG --clear --title "$TITLE1" --checklist "$MESSAGE1" 8 75 2 p "$E1" on s "$E2" on 2>"$TMP" || bailout

FILES=""
SAVEHOME=""
SAVEETC=""
CHOWNHOME=""
for i in `tr -d '"' < "$TMP"`; do
case $i in
p) SAVEHOME="yes"; CHOWNHOME="chown -R $(ls -ld $HOME | awk '{print $3"."$4}') $HOME" ;;
s) SAVEETC="yes" ;;
esac
done

# Find changes in (real) files of dir1 ... dir2
findchanged(){
if [ -d "$1" ]; then
for i in `( cd "$1"; find . -type f | sed 's,^\./,,g' | grep -v ' ' )`; do
cmp -s "$1/$i" "$2/$i" || echo "$1/$i"
done
elif [ -e "$1" ]; then
cmp -s "$1" "$2" || echo "$1"
fi
}

writescript(){
cat >"$1" <<EOT
#!/bin/sh
[ "\`id -u\`" = "0" ] || { echo "You need root privileges to modify the system!" >&2 ; exit 1; }
[ -d "\$1" ] && CONFIGS="\$1/configs.tbz"
[ -f "\$CONFIGS" ] || CONFIGS="/cdrom/KNOPPIX/configs.tbz"
[ -f "\$CONFIGS" ] || CONFIGS="/mnt/floppy/configs.tbz"
SCRIPTS=""
REINIT=""
if [ -f "\$CONFIGS" ]; then
echo "[1mExtracting config archive $CONFIGS...[0m"
for i in \`tar -jpPtf \$CONFIGS\`; do
rm -f "\$i"
case "\$i" in
/etc/network/interfaces) killall dhcpcd 2>/dev/null ; SCRIPTS="\$SCRIPTS network";;
/etc/cups/cupsd.conf) SCRIPTS="\$SCRIPTS cupsys";;
/etc/inittab) REINIT="yes";;
esac
done
tar -jpPxf "\$CONFIGS" ; $CHOWNHOME
fi
if [ -n "\$SCRIPTS" ]; then
echo "[1mStarting daemons...[0m"
for i in \$SCRIPTS; do
[ -e /etc/init.d/\$i ] && /etc/init.d/\$i start
done
fi
if [ -n "\$REINIT" ]; then
echo "[1mReloading INIT.[0m"
init q
fi
EOT
return "$?"
}

FLOPPYDIR="$(ls -l /mnt/floppy | awk '{print $NF}')"
umountfloppy(){
mount | grep -q "$FLOPPYDIR" && umount "$FLOPPYDIR" 2>/dev/null
}

while :; do
umountfloppy
$DIALOG --title "$TITLE1" --yesno "$MESSAGE2" 8 65 || bailout
echo "$FLOPPYDIR" | grep -q auto || mount /mnt/floppy
[ "$?" = "0" ] && writescript "$FLOPPYDIR/knoppix.sh" && break
done

# Just to be on the safe side
umountfloppy

gauge(){
status=0
while [ ! -e "$TMP.done" ]; do echo "$status" ; status="`expr \( 100 - $status \) / 4 + $status`"; sleep 2; done | $DIALOG --title "$TITLE1" --gauge "$MESSAGE3" 8 65 0
}


# Start status bar
if [ "$DIALOG" = "dialog" ]; then
gauge &
else
# Gauge doesn't work yet with gdialog
set -m
$DIALOG --title "$TITLE1" --gauge "$MESSAGE3" 8 65 75 &
fi

(
 [ -n "$SAVEHOME" ] && for i in $HOME/.??* $HOME/office; do findchanged "$i" /etc/skel/$(basename "$i"); done | egrep -v -e '(\.ICEauthority|ksycoka|\.xsession-errors|\.DCOP|\.MCOP|/etc/ioctl.save|office/user/work)'
 [ -n "$SAVEETC" -a -d /KNOPPIX/etc ] && findchanged /etc /KNOPPIX/etc
) | BZIP2=-9 tar -T - -cpPjf "$FLOPPYDIR/configs.tbz" 2>"$TMP"
RC="$?"

umountfloppy

# Stop status bar
[ "$DIALOG" = "dialog" ] && { touch "$TMP.done" ; wait ; rm -f "$TMP.done" ; } || { kill %% ; sleep 1; }

[ "$RC" = "0" ] && $DIALOG --title "$TITLE1" --msgbox "$SUCCESS" 10 65 || $DIALOG --title "$TITLE1" --msgbox "$ERROR `cat $TMP`" 10 75

bailout
